/*  
 *  File:       SxNano_Interface_MobaLedLib_V33.ino
 *  Autor:      Frank Keil
 *  Controller: Arduino Nano (5V, 16MHz), ATmega328
 *  Hardware:   Sx-Decoder von Frank Keil 
 *              http://frank-keil.de/Selectrix/Sx-Decoder/Sx-Decoder.html
 *  
 *  Libraries:  SXArduino: Quellen von G.v.d.Sel  
 *              https://github.com/gvandersel/SXArduino
 *
 *  Created on: 02.01.2012
 *  Last changed on: 13.06.2021
 *  
 *  Beschreibung:
 *  Es werden alle SX-Kanäle ausgelesen und wenn sich etwas auf dem Sx-Bus verändert hat
 *  wird dieser Wert konvertiert und über die RS-232 Schnittstelle geschickt.
 *  
 *  Pin2=T0=INT0, Pin4=T1, Pin3=SX-WriteHigh, Pin6=SX-WriteLow
 *
 *  Es werden die SX-Kanäle 0-103 ausgelesen und wenn sich ein Wert geändert hat,
 *  wird dieser über die RS-232 Schnittstelle geschickt, wenn der Eingang gesetzt wird.
 *  
 *  Bit_15 signalisiert die Gleisspannung
 *  Bit_16 ist während der Programmierung an
 *
 *  Daten werden wie folgt gesendet
 *  
 *  @XXX 0Y 01
 *  XXX = Adr*8+Kanal
 *  Y  = an/aus
 *  
 */
  
// Sx Teil
  #include "SXArduino.h"                      // Selectrix Bibliothek hinzufügen
  SXArduino SXbus(2,4,6,3);                   // Get access to the SX-bus
  bool noCC2000 = false;                      // false 0 ... 103, true 0 ... 111

  int analogPin = 6;                          // Progtaster
  #define SEND_DISABLE_PIN A1                 // Eingang der das senden startet
  #define Bit_15 12                           // Bit_15 Led - Gleis ein
  #define Bit_16 13                           // Bit_16 Led - Prog aktiv
  
  byte DecAdr=10;                             // Decoder Adresse
  byte SxAdr1=10;                             // Sx Adresse 1
  byte SxData1=0;                             // aktuelle SxDaten 1                  
  byte OldSxData1=6;                          // altes Byte 1
  byte SxAdrSchreib=0;                        // SxAdressen die geschrieben werden 
  byte SxDataSchreib=0;                       // SxDaten die geschrieben werden  
  byte tb;                                    // Gleisbit
  byte oldtb;                                 // alter Wert Gleisbit
  byte lastadr;                               // Last address read on SXbus
  byte i=105;                                 // Zählvariable
  byte j;                                     // Zählvariable
  byte beleg[111];                            // aktuelle Daten des Belegtmelder
  byte oldbeleg[111];                         // alte Daten des Belegtmelder
  char daten[8];                              // Daten die geschickt werden sollen
  
void sxisr(void)                              // Unterprogramm
    {
    SXbus.isr();
    }

void setup() 
    {                                         // NanoESP Teil
    Serial.begin(115200);                     // Schnittstelle setzen
    // Sx Teil  
    pinMode(SEND_DISABLE_PIN, INPUT_PULLUP);  // A1 als Eingang mit internen pullup Widerstand
    pinMode(Bit_15,OUTPUT);                   // D12 als Ausgang setzen
    pinMode(Bit_16,OUTPUT);                   // D13 als Ausgang setzen
    digitalWrite(Bit_16, 0);
    SXbus.init();                             // SX-bus Kommunikation
    attachInterrupt(0, sxisr, RISING);        // wechsel INT0 triggers the interrupt routine sxisr (see above)    
    lastadr = 104;                            // <= DEFINE HERE THE MAXIMAL ADDRESSE (JETZT 112, ALS0 0 .... 111)
    // Serial.println("start");               // serielle Ausgabe
    }

void loop()                                   // Die Arbeitsschleife
    {
    // Sx Teil                                // Sx-Adressen auslesen und auf Schnittstelle schreiben         
    tb = SXbus.readPWR();                     // Info, ob das Gleissignal an/aus ist
        if (oldtb != tb)                      // Gleisbit geändert?
            {
            oldtb = tb;                       // Gleisbit gleich setzen
            digitalWrite(Bit_15,tb);          // Led je nach Zustand ein/ausschalten
            Serial.println(tb);               // serielle Ausgabe
            }
    for (i=0; i<=lastadr; i++)                // Schleife aller Sx-Adressen
        {
        beleg[i]=SXbus.read(i);               // Abfrage der Sx-Adresse     
        if (oldbeleg[i] != beleg[i])          // hat sich der Wert geändert?
            {
            Schreiben();                      // falls ja, ins Unterprog springen
            }
        oldbeleg[i] = beleg[i];               // alt = neu setzen 
        }
    }

void Schreiben()                              // Unterprogramm
    {          
    for (j=0; j <= 7; j++)                    // die 8 Bit abfragen
        {    
        if (bitRead(oldbeleg[i],j) != bitRead(beleg[i],j))  // hat sich ein Bit geändert?
            {      
            sprintf(daten, "@%3i 0%i 01", i*8+j, 0+bitRead(beleg[i],j));  // MoBaLib Style
            // sprintf(daten, "$ %2i%i %i", i, j+1, bitRead(beleg[i],j)); // Sx Style
            while(digitalRead(SEND_DISABLE_PIN) == 1);
                {
                                              // warte auf Freigabe
                }
            Serial.println(daten);            // serielle Ausgabe
            }           
        }   
    // Serial.print("fertig ");               // serielle Ausgabe
    } 
